@@ -35,6 +35,7 @@ extern int applets_to_sync;
 extern int panels_to_sync;
 extern int need_complete_save;
 
+extern gboolean commie_mode;
 extern GlobalConfig global_config;
 extern PanelWidget *current_panel;
 
@@ -78,6 +79,7 @@ kill_applet_in_idle(gpointer data)
 	AppletInfo *info = data;
 	if (info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
+		extern_save_last_position (ext, TRUE /* sync */);
 		ext->clean_remove = TRUE;
 	} else if (info->type == APPLET_SWALLOW) {
 		Swallow *swallow = info->data;
@@ -293,6 +295,13 @@ applet_remove_callback(AppletInfo *info, const char *callback_name)
 	menu = applet_get_callback (info->user_menu, callback_name);
 	if (menu != NULL) {
 		info->user_menu = g_list_remove (info->user_menu, menu);
+		if (menu->menuitem != NULL)
+			gtk_widget_destroy (menu->menuitem);
+		menu->menuitem = NULL;
+		if (menu->submenu != NULL)
+			gtk_widget_destroy (menu->submenu);
+		menu->submenu = NULL;
+
 		g_free(menu->name);
 		menu->name = NULL;
 		g_free(menu->stock_item);
@@ -377,7 +386,7 @@ setup_an_item(AppletUserMenu *menu,
 	/* if the item is a submenu and doesn't have it's menu
 	   created yet*/
 	} else if(!menu->submenu) {
-		menu->submenu = scroll_menu_new();
+		menu->submenu = hack_scroll_menu_new();
 	}
 
 	if(menu->submenu) {
@@ -439,7 +448,7 @@ add_to_submenus (AppletInfo *info,
 	}
 	
 	if (s_menu->submenu == NULL) {
-		s_menu->submenu = scroll_menu_new();
+		s_menu->submenu = hack_scroll_menu_new();
 		/*a more elegant way to do this should be done
 		  when I don't want to go to sleep */
 		if (s_menu->menuitem != NULL) {
@@ -463,25 +472,27 @@ create_applet_menu (AppletInfo *info, gboolean is_basep)
 	GList *user_menu = info->user_menu;
 	gchar *pixmap;
 
-	info->menu = scroll_menu_new ();
+	info->menu = hack_scroll_menu_new ();
 
-	menuitem = gtk_menu_item_new();
-	setup_menuitem(menuitem,
-		       gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REMOVE),
-		       _("Remove from panel"));
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-			   (GtkSignalFunc) remove_applet_callback,
-			   info);
-	gtk_menu_append(GTK_MENU(info->menu), menuitem);
-	
-	menuitem = gtk_menu_item_new();
-	setup_menuitem(menuitem,NULL,_("Move"));
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-			   (GtkSignalFunc) move_applet_callback,
-			   info);
-	gtk_menu_append(GTK_MENU(info->menu), menuitem);
+	if ( ! commie_mode) {
+		menuitem = gtk_menu_item_new();
+		setup_menuitem(menuitem,
+			       gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REMOVE),
+			       _("Remove from panel"));
+		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+				   (GtkSignalFunc) remove_applet_callback,
+				   info);
+		gtk_menu_append(GTK_MENU(info->menu), menuitem);
 
-	panel_menu = scroll_menu_new();
+		menuitem = gtk_menu_item_new();
+		setup_menuitem(menuitem,NULL,_("Move"));
+		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+				   (GtkSignalFunc) move_applet_callback,
+				   info);
+		gtk_menu_append(GTK_MENU(info->menu), menuitem);
+	}
+
+	panel_menu = hack_scroll_menu_new();
 	make_panel_submenu (panel_menu, TRUE, is_basep);
 	menuitem = gtk_menu_item_new ();
 
