@@ -89,6 +89,7 @@ enum {
 	/*TYPE_CHANGE_SIGNAL,*/
 	MODE_CHANGE_SIGNAL,
 	STATE_CHANGE_SIGNAL,
+	SCREEN_CHANGE_SIGNAL,
 	WIDGET_LAST_SIGNAL
 };
 
@@ -138,6 +139,19 @@ basep_widget_realize (GtkWidget *w)
 		klass->realize (w);
 }
 
+static void
+basep_widget_map (GtkWidget *w)
+{
+        BasePWidget *basep = BASEP_WIDGET (w);
+
+        g_return_if_fail (IS_BASEP_WIDGET (basep));
+
+        if (GTK_WIDGET_CLASS (basep_widget_parent_class)->map != NULL)
+		GTK_WIDGET_CLASS (basep_widget_parent_class)->map (w);
+
+        basep_widget_update_winhints (basep);
+}
+
 static void
 basep_widget_size_request (GtkWidget *widget,
 			   GtkRequisition *requisition)
@@ -279,15 +293,11 @@ basep_widget_size_allocate (GtkWidget *widget,
 	widget->allocation = *allocation;
 
 	if (GTK_WIDGET_REALIZED(widget)) {
-		gdk_window_set_hints (widget->window,
-				      allocation->x, 
-				      allocation->y,
-				      0,0,0,0, GDK_HINT_POS);
-		gdk_window_move_resize (widget->window,
-					allocation->x, 
-					allocation->y,
-					allocation->width,
-					allocation->height);
+		xstuff_set_pos_size (widget->window,
+				     allocation->x, 
+				     allocation->y,
+				     allocation->width,
+				     allocation->height);
 	}
 
 	gtk_widget_size_allocate (basep->ebox, &challoc);
@@ -297,14 +307,28 @@ static void
 basep_widget_mode_change (BasePWidget *basep, BasePMode mode)
 {
 	if (IS_BORDER_WIDGET (basep))
-		basep_border_queue_recalc ();
+		basep_border_queue_recalc (basep->screen);
 }
 
 static void
 basep_widget_state_change (BasePWidget *basep, BasePState state)
 {
 	if (IS_BORDER_WIDGET (basep))
-		basep_border_queue_recalc ();
+		basep_border_queue_recalc (basep->screen);
+}
+
+static void
+basep_widget_real_screen_change (BasePWidget *basep, int screen)
+{
+	if (basep->screen != screen) {
+		basep_border_queue_recalc (basep->screen);
+		basep->screen = screen;
+		/* this will queue border recalc in the new screen */
+		gtk_widget_queue_resize (GTK_WIDGET (basep));
+		panels_to_sync = TRUE;
+
+		update_config_screen (basep);
+	}
 }
 
 static void
@@ -345,16 +369,29 @@ basep_widget_class_init (BasePWidgetClass *klass)
 			       GTK_TYPE_NONE,
 			       1, GTK_TYPE_ENUM);
 
+	basep_widget_signals[SCREEN_CHANGE_SIGNAL] = 
+		gtk_signal_new("screen_change",
+			       GTK_RUN_LAST,
+			       object_class->type,
+			       GTK_SIGNAL_OFFSET(BasePWidgetClass,
+						 screen_change),
+			       gtk_marshal_NONE__INT,
+			       GTK_TYPE_NONE,
+			       1,
+			       GTK_TYPE_INT);
+
 	gtk_object_class_add_signals(object_class, 
 				     basep_widget_signals,
 				     WIDGET_LAST_SIGNAL);
 
 	klass->mode_change = basep_widget_mode_change;
 	klass->state_change = basep_widget_state_change;
+	klass->screen_change = basep_widget_real_screen_change;
 
 	widget_class->size_request = basep_widget_size_request;
 	widget_class->size_allocate = basep_widget_size_allocate;
 	widget_class->realize = basep_widget_realize;
+	widget_class->map = basep_widget_map;
 	widget_class->enter_notify_event = basep_enter_notify;
 	widget_class->leave_notify_event = basep_leave_notify;
 	widget_class->style_set = basep_style_set;
@@ -435,7 +472,7 @@ basep_pos_get_hide_pos (BasePWidget *basep,
 	case ORIENT_DOWN:
 		*y += h - ((basep->state == BASEP_AUTO_HIDDEN)
 			   ? pw_minimized_size
-			   : get_requisition_width (basep->hidebutton_s));
+			   : get_requisition_height (basep->hidebutton_s));
 		break;
 	}
 }
@@ -687,10 +724,8 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrientType hide_orient,
 			usleep(1000);
 		}
 
-		gdk_window_resize(wid->window,dw,dh);
-		gdk_window_set_hints (wid->window,
-				      dx,dy,0,0,0,0,
-				      GDK_HINT_POS);
+		xstuff_set_pos_size (wid->window,
+				     dx, dy, dw, dh);
 		basep_widget_set_ebox_orient(basep, -1);
 	}
 	
@@ -771,12 +806,9 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrientType hide_orient,
 			(diff/1000.0)*200*(10001-(step*step));
 		
 		basep_widget_set_ebox_orient(basep, hide_orient);
-		gdk_window_resize(wid->window,ow,oh);
-		gdk_window_set_hints (wid->window,
-				      ox,oy,0,0,0,0,
-				      GDK_HINT_POS);
 		gdk_window_show(wid->window);
-		gdk_window_move(wid->window,ox,oy);
+		xstuff_set_pos_size (wid->window,
+				     ox, oy, ow, oh);
 
 		gtk_widget_show_now(wid);
 
@@ -808,10 +840,8 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrientType hide_orient,
 			usleep(1000);
 		}
 
-		gdk_window_resize(wid->window,dw,dh);
-		gdk_window_set_hints (wid->window,
-				      dx,dy,0,0,0,0,
-				      GDK_HINT_POS);
+		xstuff_set_pos_size (wid->window,
+				     dx, dy, dw, dh);
 
 		basep_widget_set_ebox_orient(basep, -1);
 	}
@@ -868,7 +898,7 @@ basep_widget_destroy (GtkObject *o)
 	basep->leave_notify_timer_tag = 0;
 
 	if (IS_BORDER_WIDGET (basep))
-		basep_border_queue_recalc ();
+		basep_border_queue_recalc (basep->screen);
 
 	gtk_object_unref (GTK_OBJECT (basep->pos));
 	basep->pos = NULL;
@@ -1073,6 +1103,13 @@ basep_widget_update_winhints (BasePWidget *basep)
 	GnomeWinLayer layer;
 	guint coverhint;
 
+        /* Do this before compliance check, the compliance
+         * check is for the legacy GNOME spec, this is for the
+         * new WM spec. Also, there's no harm to setting the hint
+         * for incompliant WM's, they'll just ignore it.
+         */
+        xstuff_set_wmspec_dock_hints (w->window);
+        
 	if ( ! basep->compliant_wm)
 		return;
 
@@ -1136,6 +1173,8 @@ void
 basep_update_frame (BasePWidget *basep)
 {
 	gboolean hide_frame = global_config.hide_panel_frame ||
+		PANEL_WIDGET (basep->panel)->back_type == 
+						PANEL_BACK_TRANSLUCENT ||
 		PANEL_WIDGET (basep->panel)->back_type == PANEL_BACK_PIXMAP;
 
 	if (hide_frame && GTK_WIDGET_VISIBLE (basep->frame)) {
@@ -1253,6 +1292,7 @@ GtkWidget*
 basep_widget_construct (BasePWidget *basep,
 			gboolean packed,
 			gboolean reverse_arrows,
+			int screen,
 			PanelOrientation orient,
 			int sz,
 			BasePMode mode,
@@ -1292,7 +1332,9 @@ basep_widget_construct (BasePWidget *basep,
 
 	gtk_widget_show(basep->panel);
 
-	if(back_type != PANEL_BACK_PIXMAP && !global_config.hide_panel_frame) {
+	if(back_type != PANEL_BACK_PIXMAP && 
+			back_type != PANEL_BACK_TRANSLUCENT &&
+			!global_config.hide_panel_frame) {
 		gtk_widget_show(basep->frame);
 		gtk_container_add(GTK_CONTAINER(basep->frame),basep->panel);
 	} else {
@@ -1346,6 +1388,8 @@ basep_widget_construct (BasePWidget *basep,
 			    GTK_SIGNAL_FUNC (basep_widget_south_clicked),
 			    basep);
 
+	basep->screen = screen;
+
 	basep->level = level;
 	basep->avoid_on_maximize = avoid_on_maximize;
 
@@ -1375,6 +1419,7 @@ basep_widget_construct (BasePWidget *basep,
 
 void
 basep_widget_change_params (BasePWidget *basep,
+			    int screen,
 			    PanelOrientation orient,
 			    int sz,
 			    BasePMode mode,
@@ -1447,9 +1492,10 @@ basep_widget_change_params (BasePWidget *basep,
 		basep_widget_update_winhints (basep);
 	}
 
+	basep_widget_set_hidebuttons (basep);
+	basep_widget_show_hidebutton_pixmaps (basep);
 
-	basep_widget_set_hidebuttons(basep);
-	basep_widget_show_hidebutton_pixmaps(basep);
+	basep_widget_screen_change (basep, screen);
 
 	gtk_widget_queue_resize (GTK_WIDGET (basep));
 }
@@ -1869,13 +1915,17 @@ basep_widget_get_menu_pos (BasePWidget *basep,
 			     x, y, wx, wy, 
 			     ww, wh);
 
-	if (*x + mreq.width > multiscreen_width (basep->screen))
-		*x = multiscreen_width (basep->screen) - mreq.width;
+	if (*x + mreq.width >
+	    multiscreen_x (basep->screen) + multiscreen_width (basep->screen))
+		*x = multiscreen_x (basep->screen) + 
+			multiscreen_width (basep->screen) - mreq.width;
 	if (*x < multiscreen_x (basep->screen))
 		*x = multiscreen_x (basep->screen);
 
-	if (*y + mreq.height > multiscreen_height (basep->screen))
-		*y = multiscreen_height (basep->screen) - mreq.height;
+	if (*y + mreq.height >
+	    multiscreen_y (basep->screen) + multiscreen_height (basep->screen))
+		*y = multiscreen_y (basep->screen) + 
+			multiscreen_height (basep->screen) - mreq.height;
 	if (*y < multiscreen_y (basep->screen))
 		*y = multiscreen_y (basep->screen);
 }
@@ -1999,13 +2049,23 @@ basep_widget_set_pos (BasePWidget *basep,
 		      int x, int y)
 {
 	int w, h;
+	int newscreen;
+	gboolean force = FALSE;
 	BasePPosClass *klass = 
 		basep_widget_get_pos_class (basep);
 
 	g_return_if_fail (klass && klass->set_pos);
 
+	/* first take care of switching screens */
+	newscreen = multiscreen_screen_from_pos (x, y);
+	if (newscreen >= 0 &&
+	    newscreen != basep->screen) {
+		force = TRUE;
+		basep_widget_screen_change (basep, newscreen);
+	}
+
 	basep_widget_get_size (basep, &w, &h);
-	klass->set_pos(basep, x, y, w, h);
+	klass->set_pos (basep, x, y, w, h, force);
 }
 
 void
@@ -2033,21 +2093,68 @@ basep_widget_set_state (BasePWidget *basep, BasePState state,
 	panels_to_sync = TRUE;
 }
 
+void
+basep_widget_screen_change (BasePWidget *basep, int screen)
+{
+	g_return_if_fail (basep != NULL);
+	g_return_if_fail (IS_BASEP_WIDGET (basep));
+	g_return_if_fail (screen >= 0);
+
+	if (basep->screen == screen)
+		return;
+
+	gtk_signal_emit (GTK_OBJECT (basep),
+			 basep_widget_signals[SCREEN_CHANGE_SIGNAL],
+			 screen);
+}
+
 /*****
  * Collision avoidance stuff
  *****/
+/* FIXME: needs to be per screen! */
 typedef struct {
 	int left;
 	int center;
 	int right;
 } Border;
 
-static Border borders[4] = {{0}};
+typedef struct {
+	int screen;
+	Border borders[4];
+	int left;
+	int right;
+	int top;
+	int bottom;
+} ScreenBorders;
+
+static GList *border_list = NULL;
+
+static ScreenBorders *
+get_borders (int screen)
+{
+	GList *li;
+	ScreenBorders *sb;
+
+	for (li = border_list; li != NULL; li = li->next) {
+		sb = li->data;
+		if (sb->screen == screen)
+			return sb;
+	}
+
+	sb = g_new0 (ScreenBorders, 1);
+	sb->screen = screen;
+	border_list = g_list_prepend (border_list, sb);
+
+	return sb;
+}
 
 static void
-basep_calculate_borders (void)
+basep_calculate_borders (int screen)
 {
 	GSList *li;
+	ScreenBorders *sb;
+
+	sb = get_borders (screen);
 
 	for (li = panel_list; li != NULL; li = li->next) {
 		PanelData *pd = li->data;
@@ -2063,7 +2170,8 @@ basep_calculate_borders (void)
 
 		basep = BASEP_WIDGET (pd->panel);
 
-		if (basep->mode == BASEP_AUTO_HIDE)
+		if (basep->mode == BASEP_AUTO_HIDE ||
+		    basep->screen != screen)
 			continue;
 
 		gtk_widget_get_child_requisition (basep->ebox, &chreq);
@@ -2074,84 +2182,98 @@ basep_calculate_borders (void)
 			BasePState state = basep->state;
 			if (PANEL_WIDGET (basep->panel)->orient ==
 			    PANEL_VERTICAL) {
-				if (borders[edge].left < chreq.width &&
+				if (sb->borders[edge].left < chreq.width &&
 				    state != BASEP_HIDDEN_RIGHT)
-					borders[edge].left = chreq.width;
-				if (borders[edge].center < chreq.width &&
+					sb->borders[edge].left = chreq.width;
+				if (sb->borders[edge].center < chreq.width &&
 				    state != BASEP_HIDDEN_RIGHT &&
 				    state != BASEP_HIDDEN_LEFT)
-					borders[edge].center = chreq.width;
-				if (borders[edge].right < chreq.width &&
+					sb->borders[edge].center = chreq.width;
+				if (sb->borders[edge].right < chreq.width &&
 				    state != BASEP_HIDDEN_LEFT)
-					borders[edge].right = chreq.width;
+					sb->borders[edge].right = chreq.width;
 			} else {
-				if (borders[edge].left < chreq.height &&
+				if (sb->borders[edge].left < chreq.height &&
 				    state != BASEP_HIDDEN_RIGHT)
-					borders[edge].left = chreq.height;
-				if (borders[edge].center < chreq.height &&
+					sb->borders[edge].left = chreq.height;
+				if (sb->borders[edge].center < chreq.height &&
 				    state != BASEP_HIDDEN_RIGHT &&
 				    state != BASEP_HIDDEN_LEFT)
-					borders[edge].center = chreq.height;
-				if (borders[edge].right < chreq.height &&
+					sb->borders[edge].center = chreq.height;
+				if (sb->borders[edge].right < chreq.height &&
 				    state != BASEP_HIDDEN_LEFT)
-					borders[edge].right = chreq.height;
+					sb->borders[edge].right = chreq.height;
 			}
 		} else /* ALIGNED */ {
 			AlignedAlignment align = ALIGNED_POS(basep->pos)->align;
 			if (PANEL_WIDGET (basep->panel)->orient ==
 			    PANEL_VERTICAL) {
 				if (align == ALIGNED_LEFT &&
-				    borders[edge].left < chreq.width)
-					borders[edge].left = chreq.width;
+				    sb->borders[edge].left < chreq.width)
+					sb->borders[edge].left = chreq.width;
 				else if (align == ALIGNED_CENTER &&
-					 borders[edge].center < chreq.width)
-					borders[edge].center = chreq.width;
+					 sb->borders[edge].center < chreq.width)
+					sb->borders[edge].center = chreq.width;
 				else if (align == ALIGNED_RIGHT &&
-					 borders[edge].right < chreq.width)
-					borders[edge].right = chreq.width;
+					 sb->borders[edge].right < chreq.width)
+					sb->borders[edge].right = chreq.width;
 			} else {
 				if (align == ALIGNED_LEFT &&
-				    borders[edge].left < chreq.height)
-					borders[edge].left = chreq.height;
+				    sb->borders[edge].left < chreq.height)
+					sb->borders[edge].left = chreq.height;
 				else if (align == ALIGNED_CENTER &&
-					 borders[edge].center < chreq.height)
-					borders[edge].center = chreq.height;
+					 sb->borders[edge].center < chreq.height)
+					sb->borders[edge].center = chreq.height;
 				else if (align == ALIGNED_RIGHT &&
-					 borders[edge].right < chreq.height)
-					borders[edge].right = chreq.height;
+					 sb->borders[edge].right < chreq.height)
+					sb->borders[edge].right = chreq.height;
 			}
 		}
 	}
 }
 
 static int
-border_max (BorderEdge edge)
+border_max (ScreenBorders *sb, BorderEdge edge)
 {
-	return MAX (borders[edge].center,
-		    MAX (borders[edge].left, borders[edge].right));
+	return MAX (sb->borders[edge].center,
+		    MAX (sb->borders[edge].left, sb->borders[edge].right));
 }
 
 void
-basep_border_recalc (void)
+basep_border_recalc (int screen)
 {
 	int i;
 	GSList *li;
-	Border old[4];
+	ScreenBorders *sb;
+	ScreenBorders old;
 
-	memcpy (old, borders, 4 * sizeof (Border));
+	sb = get_borders (screen);
+
+	memcpy (&old, sb, sizeof (ScreenBorders));
 
 	for (i = 0; i < 4; i++) {
-		borders[i].left = 0;
-		borders[i].center = 0;
-		borders[i].right = 0;
+		sb->borders[i].left = 0;
+		sb->borders[i].center = 0;
+		sb->borders[i].right = 0;
 	}
 
 	/* if not avoiding collisions, keeping things at 0 is a safe bet */
 	if (global_config.avoid_collisions) {
-		basep_calculate_borders ();
+		basep_calculate_borders (screen);
 	}
 
-	if (memcmp (old, borders, 4 * sizeof (Border)) != 0) {
+	sb->left = border_max (sb, BORDER_LEFT);
+	sb->right = border_max (sb, BORDER_RIGHT);
+	sb->top = border_max (sb, BORDER_TOP) +
+		foobar_widget_get_height (screen);
+	sb->bottom = border_max (sb, BORDER_BOTTOM);
+
+#ifdef PANEL_DEBUG
+	g_print ("[basep_border_recalc] SCREEN %d (%d %d %d %d)\n", screen,
+		 sb->left, sb->right, sb->top, sb->bottom);
+#endif
+
+	if (memcmp (&old, sb, sizeof (ScreenBorders)) != 0) {
 		for (li = panel_list; li != NULL; li = li->next) {
 			PanelData *pd = li->data;
 			GtkWidget *panel;
@@ -2160,48 +2282,74 @@ basep_border_recalc (void)
 
 			panel = pd->panel;
 
-			if (IS_BORDER_WIDGET (panel))
+			if (IS_BORDER_WIDGET (panel) &&
+			    BASEP_WIDGET (panel)->screen == screen) {
 				gtk_widget_queue_resize (panel);
+			}
 		}
-	}
 
-	/* this does not generate xtraffic if not needed */
-	xstuff_setup_desktop_area (border_max (BORDER_LEFT),
-				   border_max (BORDER_RIGHT),
-				   border_max (BORDER_TOP) +
-				     foobar_widget_get_height (0 /* FIXME */),
-				   border_max (BORDER_BOTTOM));
+		if (sb->left != old.left ||
+		    sb->right != old.right ||
+		    sb->top != old.top ||
+		    sb->bottom != old.bottom)
+			xstuff_setup_desktop_area (screen,
+						   sb->left,
+						   sb->right,
+						   sb->top,
+						   sb->bottom);
+	}
 }
 
 static guint queue_recalc_id = 0;
+static GList *recalc_list = NULL;
 
 static gboolean
 queue_recalc_handler (gpointer data)
 {
+	GList *list, *li;
+
 	queue_recalc_id = 0;
 
-	basep_border_recalc ();
+	list = recalc_list;
+	recalc_list = NULL;
+
+	for (li = list; li != NULL; li = li->next) {
+		int screen = GPOINTER_TO_INT (li->data);
+		basep_border_recalc (screen);
+	}
+
+	g_list_free (list);
 
 	return FALSE;
 }
 
 void
-basep_border_queue_recalc (void)
+basep_border_queue_recalc (int screen)
 {
+	if (g_list_find (recalc_list,
+			 GINT_TO_POINTER (screen)) == NULL)
+		recalc_list = g_list_prepend (recalc_list,
+					      GINT_TO_POINTER (screen));
 	if (queue_recalc_id == 0) {
 		queue_recalc_id = gtk_idle_add (queue_recalc_handler, NULL);
 	}
 }
 
 void
-basep_border_get (BorderEdge edge, int *left, int *center, int *right)
+basep_border_get (int screen, BorderEdge edge,
+		  int *left, int *center, int *right)
 {
+	ScreenBorders *sb;
+
+	g_assert (screen >=0);
 	g_assert (edge >=0 && edge <= 3);
 
+	sb = get_borders (screen);
+
 	if (left != NULL)
-		*left = borders[edge].left;
+		*left = sb->borders[edge].left;
 	if (center != NULL)
-		*center = borders[edge].center;
+		*center = sb->borders[edge].center;
 	if (right != NULL)
-		*right = borders[edge].right;
+		*right = sb->borders[edge].right;
 }
